/*
 *	Common.h
 *
 *	Definitions of exported function Common.dll
 *		
 *	(c) Copyright 2001 - 2002
 *		All Rights Reserved
 */

#ifndef	COMMON_H
	#define COMMON_H

	#ifndef COMMON_API
		#define COMMON_API __declspec( dllimport )
	#endif // COMMON_API

	#include "Strings.h"
	#include <Objidl.h>	 
	
	// Functions
	COMMON_API	void		diagTrace(LPCTSTR lpszFormat, ...);					
	COMMON_API	void		diagTrace6(LPCSTR strMsg, HRESULT hr);			
	COMMON_API	void		diagThrow(LPCTSTR lpszFormat, ...);					
	COMMON_API	void		diagThrow6(LPCTSTR lpszFormat, HRESULT hr);
	COMMON_API	void		diagSavePos(LPTSTR	szFile, DWORD	dwLine);	
	COMMON_API	void		diagMessageBox(LPCSTR lpszCaption, UINT uType);	
	//
	// Message box values
	#define	MSG_ERROR		MB_ICONERROR
	#define	MSG_WARNING		MB_ICONWARNING
	#define	MSG_INFO		MB_ICONINFORMATION
	//
	// Macros for debugging
	#define		DXTRACE						diagSavePos(__FILE__, __LINE__);\
											diagTrace
	#define		DXERR(str, hr)				diagSavePos(__FILE__, __LINE__);\
											diagTrace6(str, hr);
	//
	// Obsolete macros
	#define		DXTRACE0					diagSavePos(__FILE__, __LINE__);\
											diagTrace	
	#define		DXTRACE6(str, hr)			diagSavePos(__FILE__, __LINE__);\
											diagTrace6(str, hr);
	#define		DXTHROW						diagSavePos(__FILE__, __LINE__);\
											diagThrow
	#define		DXTHROWERR(str, hr)			diagSavePos(__FILE__, __LINE__);\
											diagThrow6(str, hr);	
	
	//
	// Timer values  dwTimerCommand
	#define		TIMER_START						0
	#define		TIMER_STOP						1
	#define		TIMER_GETELAPSEDTIME			2
	#define		TIMER_GETELAPSEDTIME1			7
	#define		TIMER_GETELAPSEDTIME2			8
	#define		TIMER_GETABSOLUTETIME			3
	#define		TIMER_GETAPPTIME				4
	#define		TIMER_RESET						5
	#define		TIMER_ADVANCE					6
	// Timer
	COMMON_API float	setGetTime(DWORD dwTimerCommand = TIMER_GETABSOLUTETIME);
	
	//
	// Data files support, lplpRoot must be NULL
	COMMON_API HRESULT  stgOpenDataFile3(CString strExtPath, IStorage **lplpRoot);
	COMMON_API HRESULT	stgGetFile3(CString csFullIntPath, LPBYTE* lplpData, DWORD &dwSize);
	COMMON_API HRESULT	stgCompressFile(CString csPath);
	COMMON_API HRESULT	stgUncompressFile(CString csPath, LPVOID* lplpData, DWORD *pdwSize);

	typedef void (CALLBACK* FILEPROC)(LPTSTR, UINT, LPVOID, DWORD);

	//
	// Color macros
	#define SAFE_NULL(p)			{p = NULL;}
	#define SAFE_DELETE(p)			{ if(p) { delete (p);     (p)=NULL; } }
	#define SAFE_RELEASE(p)			{ if(p) { (p)->Release(); (p)=NULL; } }
	#define SAFE_DELETE_ARRAY(p)	{ if(p) { delete[] (p);   (p)=NULL; } }
	// TEXTCOLOR macro
	// Used to pass color of text (32-bit)
	#define TEXTCOLOR(r, g, b) ((DWORD) (((BYTE) (b) | \
		((WORD) (g) << 8)) | \
		(((DWORD) (BYTE) (r)) << 16)))
	// SURFACECOLOR macro
	// Used to pass color fo solid surfaces (16-bit)
	#define	SURFACECOLOR(r, g, b) ((DWORD) (((BYTE) (b) | \
		((WORD) (g) << 5)) | \
		(((DWORD) (BYTE) (r)) << 11)))
	//
	// Ini file strings
	//
	// Sections
	#define		_S_SECTION_GRAPHICS			_T("Graphics")
	#define		_S_SECTION_GAME				_T("Game")
	#define		_S_SECTION_DEBUG			_T("Debug")
	//
	// Common funcions
	COMMON_API int		setGetSetupInt(CString strSection, CString strKey);
	COMMON_API HRESULT	setWriteSetupString(CString strSection, CString strKey, CString strString);
	COMMON_API HRESULT	setWriteSetupInt(CString strSection, CString strKey, int nValue);
	COMMON_API CString  	setGetSetupString(CString strSection, CString strKey);
	COMMON_API CString 	setGetProgramPath();
	COMMON_API CString  	setGetSetupFilePath();
	COMMON_API CString  	setCutDirectory(CString strFullPath);
	COMMON_API CString  	setCutFile(CString strFullPath);
	COMMON_API CString  	setGetDataFilePath(CString strDataFile);		
	//
	// Colors
	//
	// Color constants for text
	#define		TC_YELLOW					TEXTCOLOR(255,255,0)
	#define		TC_BLACK					TEXTCOLOR(0,0,0)
	#define		TC_WHITE					TEXTCOLOR(255,255,255)
	#define		TC_RED						TEXTCOLOR(255,0,0)
	#define		TC_BLUE						TEXTCOLOR(0,0,255)
	#define		TC_GREEN					TEXTCOLOR(0,255,0)
	#define		TC_DARKGREEN				TEXTCOLOR(0,150,0)
	//
	// Color constants for surfaces
	#define		SC_YELLOW					SURFACECOLOR(255,255,0)
	#define		SC_RED						SURFACECOLOR(255,0,0)
	#define		SC_GREEN					SURFACECOLOR(0,255,0)
	#define		SC_BLUE						SURFACECOLOR(0,0,255)
	#define		SC_WHITE					SURFACECOLOR(255,255,255)
	#define		SC_BLACK					SURFACECOLOR(0,0,0)

	//
	// Ini file strings
	#define		_S_SECTION_COMMON			_T("Common")
	#define		_S_KEY_TRACETOFILE			_T("TraceToFile")


#endif	// COMMON_H