// Display.cpp: implementation of the CDisplay class.
//
#include "stdafx.h"

#define DISPLAY_API __declspec(dllexport)


#include "Common.h"
#include "core.h"
#include "DDFontEngine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
CDisplay::CDisplay()
{
	SAFE_NULL(m_lpDD);
	SAFE_NULL(m_lpDDPalette);
	SAFE_NULL(m_lpDDClipper);

	m_CurGamma		= 256; 
	m_bIsInit		= false;

	m_Frames    = 99;		
	m_StopTime  = 0;
	m_StartTime = 0;
	
	m_dwFlags       = 0;

}

CDisplay::~CDisplay()
{
	Clean();	
}

/*
 *	Clean all after initialization
 */
void CDisplay::Clean()
{
	if(m_bIsInit)  { 
		DXERR("Restoring display: ", m_lpDD->RestoreDisplayMode());
		SAFE_RELEASE(m_lpDDClipper);
		SAFE_RELEASE(m_lpDDPalette);
        SAFE_RELEASE(m_lpDD);
		m_dwFlags		= 0;
		m_bIsInit		= false;
	}
}

/*
 *	Initialization function, set display mode and create fliping loop
 *
 *	hWnd is handle to main window
 *  dwFlags is creation flags:
 *  DDFLG_WINDOW		- currenly not supported
 *	DDFLG_FULLSCREEN	- create fullscreen display scene
 *	DDFLG_CLIPPER		- create clipper to the scene
 *	DDFLG_PALETTE		- set palette to back buffer, but only if is set 8-bit color
 *  DDFLG_GAMMA			- add gamma correction support, maybe not supported on all systems
 */
HRESULT CDisplay::CreateFullScreenSystem(HWND hWnd, DWORD dwFlags)
{	
	//
	// Temporary front and back buffer
	LPDIRECTDRAWSURFACE7	lpDDSBack, lpDDSFront;
	DDCAPS	Hel;		Hel.dwSize		= sizeof(DDCAPS);		
	DDCAPS  Driver;		Driver.dwSize	= sizeof(DDCAPS);				
	HRESULT dwRet;

	//
	// Init olny if we not init before
	if(m_bIsInit) {
		return 1;
	}
	//
	// Check handle to window
	ASSERT(hWnd);
	//
	// Get resolution from "setup.ini" file
	m_Res_X  = setGetSetupInt(_S_SECTION_GRAPHICS, _S_KEY_WIDTH);
	m_Res_Y  = setGetSetupInt(_S_SECTION_GRAPHICS, _S_KEY_HEIGHT);
	m_Depth  = setGetSetupInt(_S_SECTION_GRAPHICS, _S_KEY_DEPTH);
	if(m_Res_X == 0) m_Res_X = _C_DEFAULT_SCREENWIDTH;
	if(m_Res_Y == 0) m_Res_Y = _C_DEFAULT_SCREENHEIGHT;
	if(m_Depth == 0) m_Depth = _C_DEFAULT_SCREENDEPTH;

	if(setGetSetupInt(_S_SECTION_GRAPHICS, _S_KEY_FPS)) {
		m_dwFlags |= DDFLG_FPS;
	}
	//
	// Set scroll rect 
	m_ScrollRect.SetRect(0, 0, m_Res_X, m_Res_Y);		
	//
	DXTRACE("Initializing display...");		
	// Creating DDraw object
	dwRet = DirectDrawCreateEx(NULL, (void**)&m_lpDD, IID_IDirectDraw7, NULL);
	DXERR ("Creating DirectDraw object", dwRet);
	if(dwRet != DD_OK) {			
		Clean();
		return dwRet;
	}	
	//
	// Get capabilities of hardware
	dwRet = m_lpDD->GetCaps(&Driver, &Hel);
	DXERR ("Getting caps about hardware", dwRet);
	if(dwRet != DD_OK) {			
		Clean();
		return dwRet;
	}	
	
	// Set cooperative level	
	dwRet = m_lpDD->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN);
	DXERR ("Setting exclusive mode", dwRet);
	if(dwRet != DD_OK) {			
		Clean();
		return dwRet;
	}		
	// Set display mode	
	dwRet = m_lpDD->SetDisplayMode(m_Res_X, m_Res_Y, m_Depth, 0, 0);
	DXERR("Setting display mode", dwRet);
	if(dwRet != DD_OK) {			
		Clean();
		return dwRet;
	}	

	//
	//	Create flipping loop
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize            = sizeof(ddsd);
	ddsd.dwFlags           = DDSD_CAPS|DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps    = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP |
                             DDSCAPS_COMPLEX | DDSCAPS_3DDEVICE;
	ddsd.dwBackBufferCount = 1;
	//
	// Get a pointer to the front buffer
	dwRet = m_lpDD->CreateSurface(&ddsd, &lpDDSFront, NULL);
	DXERR ("Getting pointer to front buffer", dwRet);
	if(dwRet != DD_OK) {			
		Clean();
		return dwRet;
	}	
	//
	// Create front buffer
	dwRet = m_surFront.Create(lpDDSFront);
	DXERR ("Creating front buffer", dwRet);
	if(dwRet != DD_OK) {
		Clean();
		return dwRet;
	}		
	//
	// Get a pointer to the back buffer
	DDSCAPS2 ddscaps;
	ZeroMemory(&ddscaps, sizeof(ddscaps));
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;				
	dwRet = m_surFront.GetSurface()->GetAttachedSurface(&ddscaps, &lpDDSBack);
	if(dwRet != DD_OK) {
		DXERR ("Getting pointer to back buffer", dwRet);
		Clean();
		return dwRet;
	}
	//
	// Create back buffer
	dwRet = m_surBack.Create(lpDDSBack);
	if(dwRet != DD_OK) {
		DXERR ("Creating back buffee", dwRet);
		Clean();
		return dwRet;
	}		
	////////////////////////////////////////////////////////////////////////
	m_bIsInit = true;

	dwRet = fntCreateFont(_S_ARIAL_DDF, _S_FONT_ARIAL);
	if(dwRet != ERROR_SUCCESS) {
		DXERR("Cannot create specified font due", dwRet);
	}	
	dwRet = fntCreateFont(_S_HUMANS_DDF, _S_FONT_HUMANS, TC_DARKGREEN);
	if(dwRet != ERROR_SUCCESS) {
		DXERR("Cannot create specified font due", dwRet);
	}	
	//
	// Display loading bitmap
	disDefineBackground(_S_LOADING1, DDFLG_VIDEO_MEMORY);
	disUpdateBackground();	
	disPresent();	
	//
	//	Clipper	
	if(dwFlags & DDFLG_CLIPPER) {
		//
		// Create rectangular region
		CRgn rect; RGNDATA data; int n;
		rect.CreateRectRgnIndirect(CRect(0, 0, m_Res_X, m_Res_Y));			
		n = rect.GetRegionData(NULL, 0);
		if(rect.GetRegionData(&data, n) != ERROR) {	
			//
			// Create clipper
			dwRet = m_lpDD->CreateClipper(0, &m_lpDDClipper,  NULL);				
			DXERR ("Creating clipper object", dwRet);
			if(DD_OK != dwRet) {					
				Clean();
				return dwRet;					
			}	
			//
			// Set clipper			
			dwRet = m_lpDDClipper->SetClipList(&data, 0);				
			DXERR ("Setting clipper list", dwRet);
			if(DD_OK != dwRet) {					
				Clean();
				return dwRet;					
			}	
			dwRet = m_surBack.GetSurface()->SetClipper(m_lpDDClipper);
			DXERR ("Setting clipper", dwRet);
			if(DD_OK != dwRet) {					
				Clean();
				return dwRet;					
			}
			m_dwFlags |= DDFLG_CLIPPER;			
		}
		else {
			Clean();
			return 1;
		}	
	}
	//
	//	Gamma correction
	BOOL bUseGamma = setGetSetupInt(_S_SECTION_GRAPHICS, _S_KEY_USEGAMMA);
	if(bUseGamma && (dwFlags & DDFLG_GAMMA) && HasGammaSupport()) {
		dwRet = m_surFront.GetSurface()->QueryInterface(IID_IDirectDrawGammaControl, (LPVOID*) &m_GammaControl);
		DXTRACE6("Getting pointer to gamma correction object due", dwRet);
		if(!m_GammaControl) {					
			Clean();
			return dwRet;
		}						
		m_dwFlags |= DDFLG_GAMMA;
	}		
	//		
	// Create and set palette
	// If we have only 8-bit color depth, we need palette
	if(m_Depth == 8) {
		dwRet = SetPalette();
		DXERR("Setting pallete due", dwRet);
		if(dwRet != DD_OK) {				
			Clean();
			return dwRet;
		}
		m_dwFlags |= DDFLG_PALETTE;
	}


	
	return dwRet;
}

//-----------------------------------------------------------------------------
// Name: HasGammaSupport()
// Desc: Returns TRUE if the device supports gamma control, FALSE otherwise
//-----------------------------------------------------------------------------
BOOL CDisplay::HasGammaSupport()
{
    // Get driver capabilities to determine gamma support.
    DDCAPS  ddcaps;    
    ZeroMemory( &ddcaps, sizeof(ddcaps) );
    ddcaps.dwSize = sizeof(ddcaps);

    GetDirectDraw()->GetCaps( &ddcaps, NULL );
    
    // Does the driver support gamma? 
    // The DirectDraw emulation layer does not support overlays
    // so gamma related APIs will fail without hardware support.  
    if( ddcaps.dwCaps2 & DDCAPS2_PRIMARYGAMMA )
        return TRUE;
    else
        return FALSE;    
}


/*
 *		Flip surfaces
 *		Draw FPS counter to back surface
 *		Restore surfaces (Back and front)
 *		Restore background surface
 */
void CDisplay::Present()
{
	HRESULT dwResult;
	//
	// Check initialization
	if(!m_bIsInit) {
		return;
	}
	//
	// Check if we want to draw FPS
	if(m_dwFlags & DDFLG_FPS) {
		//
		// Draw FPS
		dwResult = UpdateFPS();
		if(dwResult != ERROR_SUCCESS) {
			DXERR("Cannot UpdateFPS due", dwResult);
		}		
	}
	//
	// Try to flip surfaces
	// Neverending loop
	while(1)
	{
		dwResult = m_surFront.GetSurface()->Flip(NULL, 0);
		//we must restore main buffers
		if(dwResult == DDERR_SURFACELOST) {
			DXTRACE("Restoring surfaces");
			m_surFront.Restore();
			m_surBack.Restore();
			if(m_surBackground.IsValid()) {
				m_surBackground.Restore(TRUE);	
			}
		}
		if(DD_OK == dwResult) {
			break;
		}
	}	
}

/*
 *	Set palette on front surface
 *	Palette is set if is used 8-bit mode
 */
HRESULT CDisplay::SetPalette()
{
    int                 i;
    int                 n;
    int                 fh;
    HRSRC               h;
    LPBITMAPINFOHEADER  lpbi;
    PALETTEENTRY        ape[256];
    RGBQUAD *           prgb;
	CString szBitmap = "PALETA";
	HRESULT				dwReturn;
    //
    // build a 332 palette as the default.
    //
    for (i=0; i<256; i++)
    {
		ape[i].peRed   = (BYTE)(((i >> 5) & 0x07) * 255 / 7);
		ape[i].peGreen = (BYTE)(((i >> 2) & 0x07) * 255 / 7);
		ape[i].peBlue  = (BYTE)(((i >> 0) & 0x03) * 255 / 3);
		ape[i].peFlags = (BYTE)0;
    }
    //
    // get a pointer to the bitmap resource.
    //
    if (szBitmap && (h = FindResource(NULL, szBitmap, RT_BITMAP)))  {
		//////////////////////////////////////////////////////////////
		lpbi = (LPBITMAPINFOHEADER)LockResource(LoadResource(NULL, h));
		if (!lpbi) {
			DXTRACE("Lock resource failed.");
		}			
		prgb = (RGBQUAD*)((BYTE*)lpbi + lpbi->biSize);
		if (lpbi == NULL || lpbi->biSize < sizeof(BITMAPINFOHEADER))
			n = 0;
		else if (lpbi->biBitCount > 8)
			n = 0;
		else if (lpbi->biClrUsed == 0)
			n = 1 << lpbi->biBitCount;
		else
			n = lpbi->biClrUsed; 
		//
		//  a DIB color table has its colors stored BGR not RGB
		//  so flip them around.
		//
		for(i=0; i<n; i++ )	{
			ape[i].peRed   = prgb[i].rgbRed;
			ape[i].peGreen = prgb[i].rgbGreen;
			ape[i].peBlue  = prgb[i].rgbBlue;
			ape[i].peFlags = 0;
		}
    }
    else {
		if (szBitmap && (fh = _lopen(szBitmap, OF_READ)) != -1)  {
			BITMAPFILEHEADER bf;
			BITMAPINFOHEADER bi;

			_lread(fh, &bf, sizeof(bf));
			_lread(fh, &bi, sizeof(bi));
			_lread(fh, ape, sizeof(ape));
			_lclose(fh);

			if (bi.biSize != sizeof(BITMAPINFOHEADER))
				n = 0;
			else if (bi.biBitCount > 8)
				n = 0;
			else if (bi.biClrUsed == 0)
				n = 1 << bi.biBitCount;
			else
				n = bi.biClrUsed;
			//
			//  a DIB color table has its colors stored BGR not RGB
			//  so flip them around.
			//
			for(i=0; i<n; i++ )	{
				BYTE r = ape[i].peRed;
				ape[i].peRed  = ape[i].peBlue;
				ape[i].peBlue = r;
			}
		}
		else {			
			DXTRACE("Cannot find bitmap for palette.");
		}
	}
    dwReturn = m_lpDD->CreatePalette(DDPCAPS_8BIT, ape, &m_lpDDPalette, NULL);
	DXERR("Creating palette", dwReturn);			
	if(DD_OK != dwReturn) {
		DXERR("Cannot create palette due", dwReturn);
		return dwReturn;		
	}
	dwReturn = m_surFront.GetSurface()->SetPalette(m_lpDDPalette);
	DXERR("Setting palette", dwReturn);			
	if(DD_OK != dwReturn) {
		DXERR("Cannot set palette due", dwReturn);
		return dwReturn;		
	}	
	return dwReturn;
}


/*
 *	Define solid color background
 *
 *	crColor parameter defines color of background
 *  Always returns ERROR_SUCCESS
 */
HRESULT	CDisplay::DefineBackground(COLORREF	crColor)
{	
	//
	// Release old surface if initialized
	m_surBackground.Release();	
	//
	// Check initialization
	if(!m_bIsInit) {		
		return 1;
	}
	//
	// Save color of background
	m_BackColor				= crColor;	
	//
	// Enable solid background
	m_dwFlags				|= DDFLG_SOLID;
	//
	// Disable scrolling
	m_dwFlags				&= ~DDFLG_SCROLLING;
	return ERROR_SUCCESS;	
}

/*
 *	Defines color bitmap background
 *
 *	strBMPFile defines internal path of background bitmap
 *	nFlags defines properties of background:
 *	DDFLG_SCROLLING		- creates scrolling background
 *	DDFLG_VIDEO_MEMORY	- creates background in video memory
 *  DDFLG_SYSTEM_MEMORY - creates background in RAM
 *
 *	Default is Video memory, if is not enough video memory, it is used RAM
 *  Previous background is released 
 */
HRESULT	CDisplay::DefineBackground(CString strBMPFile, DWORD dwFlags)
{
	HRESULT dwRet;
	DDSURFACEDESC2 ddsdesc;
	// Init size before use	 
	ddsdesc.dwSize	= sizeof(DDSURFACEDESC2);
	//
	// Check initialization
	if(!m_bIsInit) {
		return 1;
	}
	//
	// Release old surface if initialized
	m_surBackground.Release();	
	//
	// Try to create new surface for background bitmap
	dwRet = m_surBackground.Create(m_Res_X, m_Res_Y, dwFlags);
	DXERR("Creating surface for background due", dwRet);
	if(dwRet != ERROR_SUCCESS) {				
		return dwRet;
	}
	//
	// Copy bitmap to surface
	dwRet = m_surBackground.CopyBitmap(strBMPFile);
	DXERR("Copying bitmap to surface due", dwRet);
	if(dwRet != ERROR_SUCCESS) {				
		return dwRet;
	}
	// 
	// Save info about character of background	
	if(dwFlags & DDFLG_SCROLLING) {
		m_dwFlags |= DDFLG_SCROLLING;
	}
	else {
		m_dwFlags &= ~DDFLG_SCROLLING;
	}
	//
	// Save info about background
	// Save internal path for restore 
	m_BackBMP			= strBMPFile;
	// No solid, we want bitmap background
	m_dwFlags &= ~DDFLG_SOLID;
	//
	// Init source rectangle for blitting
	m_BackSource.left   = 0;					
	m_BackSource.top    = 0;					
	m_BackSource.right  = m_surBackground.Width();
	m_BackSource.bottom = m_surBackground.Height();		
	return dwRet;
}

/*
 *	Update scrolling/notscrolling background
 *
 *	Note: Call it as first in UpdateFrame()
 *
 */
HRESULT	CDisplay::UpdateBackground()
{
	HRESULT dwReturn;
	CRect dest1, src1, dest2, src2;
	//
	// Detect if background is scrolling or not
	if(m_dwFlags & DDFLG_SCROLLING) {
		//
		// Moving of blitting area
		m_ScrollRect.left += 1;
		m_ScrollRect.right += 1;
		//
		// Reset m_rectScroll , 
		// End of the screen: m_ScrollRect.left = 0, m_ScrollRect.right = m_Res_X;
		if(m_ScrollRect.left == (int)m_Res_X) {
			m_ScrollRect.left = 0;
			m_ScrollRect.right = m_Res_X;
		}		
		//
		// Compute of right and left parts of blitting source anf destination rectangles
		//
		// Left destination rectangle
		dest1.top    = 0;
		dest1.left   = 0;
		dest1.right  = m_Res_X - m_ScrollRect.left;
		dest1.bottom = m_Res_Y;
		//
		// Left source rectangle
		src1.top     = 0;
		src1.left    = m_ScrollRect.right - m_Res_X;
		src1.right   = m_Res_X;
		src1.bottom  = m_Res_Y;
		//
		// Right destination rectangle
		dest2.top    = 0;
		dest2.left   = m_Res_X - m_ScrollRect.left;
		dest2.right  = m_Res_X;
		dest2.bottom = m_Res_Y;
		//
		// Right source rectangle
		src2.top     = 0;
		src2.left    = 0;
		src2.right   = m_ScrollRect.right - m_Res_X;
		src2.bottom  = m_Res_Y;
		//
		// Blit both part of background to the BS
		//
		// Blit first part of bcg
		if(src1.left != src1.right) {
			dwReturn = m_surBack.Blt(dest1, src1, &m_surBackground);
			if(dwReturn != ERROR_SUCCESS) {
				DXERR("Cannot blit bitmap scrl part 1 background due", dwReturn);
				return dwReturn;
			}
		}
		//
		// Blit second part of bcg
		if(src2.left != src2.right) {
			dwReturn = m_surBack.Blt(dest2, src2, &m_surBackground);
			if(dwReturn != ERROR_SUCCESS) {
				DXERR("Cannot blit bitmap scrl part 2 background due", dwReturn);
				return dwReturn;
			}
		}	
	}
	else {
		if(m_dwFlags & DDFLG_SOLID) {
			//
			// Blit solid nonscrolling background to the back buffer
			dwReturn = m_surBack.Blt(CRect(0, 0, m_Res_X, m_Res_Y), m_BackColor);
			if(dwReturn != ERROR_SUCCESS) {
				DXERR("Cannot blit solid background due", dwReturn);
				return dwReturn;
			}
		}
		else {
			//
			// Blit bitmap non scrolling background to the BS 
			dwReturn = m_surBack.Blt(CRect(0, 0, m_Res_X, m_Res_Y), m_BackSource, &m_surBackground);
			if(dwReturn != ERROR_SUCCESS) {
				DXERR("Cannot blit bitmap background due", dwReturn);
				return dwReturn;
			}
		}		
	}
	return dwReturn;
}

HRESULT		CDisplay::IncrementGamma(int _Amount)
{
	DWORD dwRet;
	DDGAMMARAMP ramp;

	if(m_dwFlags & DDFLG_GAMMA) {	
		
		m_CurGamma += _Amount;
		if(m_CurGamma > 256) {
			m_CurGamma = 256; 
			return ERROR_SUCCESS;
		}

		ZeroMemory( &ramp, sizeof(ramp) );

		dwRet = m_GammaControl->GetGammaRamp( 0, &ramp);
		if(dwRet != DD_OK) {		
			return dwRet;
		}
		

		WORD    dwGamma = 0;
		m_CurGamma += _Amount;
		if(m_CurGamma > 256) m_CurGamma = 256;

		for( int iColor = 0; iColor < 256; iColor++ )
		{
			ramp.red[iColor]   = dwGamma;
			ramp.green[iColor] = dwGamma;
			ramp.blue[iColor]  = dwGamma;

			dwGamma += (WORD) m_CurGamma;
		}

		dwRet = m_GammaControl->SetGammaRamp( 0, &ramp );			
		if(dwRet != DD_OK) {		
			return dwRet;
		}
	}
	return 1;//ERROR_NOGAMMA;
}

HRESULT		CDisplay::DecrementGamma(int _Amount)
{
	DWORD dwRet;
	DDGAMMARAMP ramp;

	if(m_dwFlags & DDFLG_GAMMA) {	
		
		ZeroMemory( &ramp, sizeof(ramp) );

		dwRet = m_GammaControl->GetGammaRamp( 0, &ramp);
		if(dwRet != DD_OK) {
			return dwRet;
		}		

		WORD    dwGamma = 0;
		m_CurGamma -= _Amount;
		if(m_CurGamma < 0) m_CurGamma = 0;

		for( int iColor = 0; iColor < 256; iColor++ )
		{
			ramp.red[iColor]   = dwGamma;
			ramp.green[iColor] = dwGamma;
			ramp.blue[iColor]  = dwGamma;

			dwGamma += (WORD) m_CurGamma;
		}

		dwRet = m_GammaControl->SetGammaRamp( 0, &ramp );			
		if(dwRet != DD_OK) {
			return dwRet;
		}
	}
	return 1;//ERROR_NOGAMMA;
}


/*
 *	Updates pointer of FPS in left top corner.
 *
 *  Returns ERROR_SUCCESS if success else error code
 *	of DrawText() function. 
 *
 *	Note: This is internal function:
 *  do not call it from external modules.
 */
HRESULT	CDisplay::UpdateFPS()
{	
	DWORD dwRet;

	//
	// Show video memory
	if(setGetSetupInt(_S_SECTION_GRAPHICS, _S_KEY_SHOWVIDMEM)) {

		DDCAPS Driver;
		ZeroMemory(&Driver, sizeof(DDCAPS));
		Driver.dwSize	= sizeof(DDCAPS);		
		//
		// Get capabilities of hardware
		dwRet = m_lpDD->GetCaps(&Driver, NULL);		
		if(dwRet != DD_OK) {			
			DXERR ("Getting caps about hardware", dwRet);
			Clean();
			return dwRet;
		}	
		CString csTMem, csFMem;
		csTMem.Format("Total video memory: %d kB", Driver.dwVidMemTotal / 1024);
		csFMem.Format("Free video memory: %d kB", Driver.dwVidMemFree / 1024);

		fntDrawText(0, 20, csTMem, _S_FONT_ARIAL);
		fntDrawText(0, 40, csFMem, _S_FONT_ARIAL);
	}
	//
	// Get time of system start
	m_StartTime = GetTickCount();		
	//
	// Increment frames counter	
	m_Frames++;							
	//
	// Compute number of frames per one second
	if((m_StartTime - m_StopTime) >= FPS_REFRESH_RATE) {
		m_strFrames.Format("%2.1f", m_Frames * 1000 / (m_StartTime - m_StopTime));
		m_Frames = 0;
		//
		// Save old-new time
		m_StopTime = m_StartTime;		
	}
	//
	// Draw FPS to the screen
	return fntDrawText(0, 0, m_strFrames, _S_FONT_ARIAL);
}
