// GButton.cpp: implementation of the CGButton class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "..\Display\Common.h"
#include "..\Input\input1.h"
#include "..\Common\Common.h"

#include "GButton.h"
#include "gpage.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGButton::CGButton()
{
	SetCBFocused(TRUE);
}

CGButton::~CGButton()
{

}

void	CGButton::CreateButton(int x, int y, LPCSTR szNormal, LPCSTR szFocus, LPCSTR szPress, LPCSTR szDis)
{
	//
	// Create and defines segment for normal state of button - clear button
	m_NormalState.CreateState(BS_NORMAL, szNormal);
	AddState(&m_NormalState);
	//
	// Create and defines segment for focused state of button - focused button
	if(szFocus) {
		m_FocusState.CreateState(BS_FOCUS, szFocus);
		AddState(&m_FocusState);				
	}
	//
	// Create and defines segment for pressed state of button - pressed button
	if(szPress) {
		m_PressState.CreateState(BS_PRESS, szPress);
		AddState(&m_PressState);		
	}
	//
	// Create and defines segment for disabled state of button - disabled button
	if(szDis) {
		m_DisState.CreateState(BS_DISABLE, szDis);
		AddState(&m_DisState);		
	}
	//
	// Center position of button 
	if(m_pCurState) {
		if(x == IP_CENTER) {
			x = disGetResolution().cx / 2 - m_pCurState->GetSourceSurface()->Width() / 2;
		}
		if(y == IP_CENTER) {
			y = disGetResolution().cy / 2 - m_pCurState->GetSourceSurface()->Height() / 2;
		}
	}
	SetPosition(CPoint(x, y));
}

void CGButton::UpdateItem()
{
	if(GetState()->GetID() == BS_DISABLE) {
		SetCBFocused(FALSE);
	}
	else {
		SetCBFocused(TRUE);
	}

	CGItem::UpdateItem();	
}

HRESULT	CGButton::ProcessItem(CGPage* _Page, void* _Data, UINT _Action)
{
	if(GetState()->GetID() != BS_DISABLE) {

		switch(_Action) {
		case IA_MOUSEMOVE:
			if(GetState()->GetID() != BS_PRESS) {
				SetState(BS_FOCUS);
			}
			break;
		case IA_MOUSECLICK_UP:
			if((*((UINT*)_Data)) == LEFT_MOUSE_BUTTON)  {
				SetState(BS_FOCUS);		
			}
			break;
		case IA_MOUSECLICK_DOWN:		
			if((*((UINT*)_Data)) == LEFT_MOUSE_BUTTON) {
				SetState(BS_PRESS);
			}
			break;
		case IA_KEYPRESS:
			//none handling
			break;
		case IA_NONE:
				SetState(BS_NORMAL);
			break;
		}
	}

	return CGItem::ProcessItem(_Page, (void*)_Data, _Action);	
}

