// GMenu.cpp: implementation of the CGMenu class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#define   MENU_API __declspec(dllexport)

#include "..\Common\Common.h"
#include "GMenu.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/////////////////////////////////
//Only one object of menu
CGMenu theMenu;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGMenu::CGMenu()
{
	m_bInit = FALSE;
	m_pCurrentPage = NULL;	 
}

CGMenu::~CGMenu()
{
	ReleaseMenu();
}


void		CGMenu::ReleaseMenu()
{
	m_bInit = FALSE;
	m_pCurrentPage = NULL;

	CGPage *pPage;
	DXTRACE("Menu has %d page(s).", m_Pages.GetSize());
	for(int i = 0; i < m_Pages.GetSize(); i++) {
		pPage = (CGPage*) m_Pages[i];
		DXTRACE("Deleting page. ID: %d\tPointer: 0x%X", pPage->GetID(), int(pPage));
		SAFE_DELETE(pPage);
	}
	//
	// Remove pages from array
	m_Pages.RemoveAll();
}

HRESULT		CGMenu::InitMenu(MENUPROC	_ProcessFunc) 
{		
	DXTRACE("Initializing menu...");
	//
	// Menu is initilized
	m_bInit = TRUE;
	// Callback menu function is OK.
	return DefineProcessFunc(_ProcessFunc);	
}

HRESULT		CGMenu::DefineProcessFunc(MENUPROC	_ProcessFunc) 
{	
	//
	// Save pointer to call back function of menu
	ProcessFunc = _ProcessFunc;
	// Check function
	if(ProcessFunc((void*) NULL,  0, 0) != ERROR_SUCCESS) {
		DXTHROW("Menu process function is not valid. Must return ERROR_SUCCESS(0).");
	}
	return ERROR_SUCCESS;
}

void		CGMenu::CreatePage(DWORD dwID, CGPage *_NewPage) 
{		
	//
	// Check initialization of menu
	if(!m_bInit) {
		DXTHROW("Menu is not initialzed. Call menInitMenu() first.");
	}
	// Check input parametr
	if(_NewPage == NULL) {
		DXTHROW("NULL page passed.");
	}	
	// Check if the page is not already in the page array
	CGPage *pRet;
	for(int i = 0; i < m_Pages.GetSize(); i++) {
		pRet = (CGPage*) m_Pages[i];
		if(pRet->GetID() == dwID) {
			DXTHROW("Page is already in the menu system.");
		}
	}
	//
	// Set some atrributes of the page
	_NewPage->SetID(dwID);
	_NewPage->ProcessFunc = ProcessFunc;	
	//
	// Add page
	m_Pages.Add(_NewPage);	
	//
	// Show adrress fo the new page
	DXTRACE("Creating page. ID: %d\tPointer: 0x%X", dwID, int(_NewPage));
	//
	// Set current page if is not page is set
	if(m_pCurrentPage == NULL) {
		m_pCurrentPage = _NewPage;
	}
}


CGPage*		CGMenu::GetPage(DWORD dwID)
{
	//
	// Check initialization of menu
	if(!m_bInit) {
		DXTHROW("Menu is not initialzed. Call menInitMenu() first.");
	}
	CGPage *pRet = NULL;
	//
	// Find page in the array and return pointer at it
	for(int i = 0; i < m_Pages.GetSize(); i++) {
		pRet = (CGPage*) m_Pages[i];
		// Check page's IDs
		if(pRet->GetID() == dwID) {
			break;
		}
	}
	return pRet;	
}

void		CGMenu::UpdateMenu() 
{
	//
	// Check initialization of menu
	if(!m_bInit) {
		DXTHROW("Menu is not initialzed. Call menInitMenu() first.");
	}
	//
	// If some page is selected, update this page
	if(m_pCurrentPage) {
		m_pCurrentPage->UpdatePage();	
	}
}

void		CGMenu::TestMouseMove(CPoint _Cursor) 
{
	//
	// Check initialization of menu
	if(!m_bInit) {
		DXTHROW("Menu is not initialzed. Call menInitMenu() first.");
	}
	if(m_pCurrentPage) {
		m_pCurrentPage->TestMouseMove(_Cursor);
	}
}

void		CGMenu::TestMouseClick(CPoint _Cursor, UINT		_MouseButton, UINT _ButtonAction) 
{
	//
	// Check initialization of menu
	if(!m_bInit) {
		DXTHROW("Menu is not initialzed. Call menInitMenu() first.");
	}
	if(m_pCurrentPage) {
		m_pCurrentPage->TestMouseClick(_Cursor, _MouseButton, _ButtonAction);
	}
}

void	CGMenu::SetCurrentPage(DWORD dwID) 
{
	//
	// Check initialization of menu
	if(!m_bInit) {
		DXTHROW("Menu is not initialzed. Call menInitMenu() first.");
	}
	CGPage *pRet = NULL;
	//
	// Set normal states for all items
	m_pCurrentPage->ResetPage();
	// Find page in the array
	for(int i = 0; i < m_Pages.GetSize(); i++) {
		pRet = (CGPage*) m_Pages[i];
		// Check IDs
		if(pRet->GetID() == dwID && !(pRet == m_pCurrentPage)) {
			// Set current page
			m_pCurrentPage  = pRet;
			return;
		}
	}	
	DXTRACE("Specified page is not defined");
}


HRESULT	 menInitMenu(MENUPROC	_ProcessFunc)
{
	return theMenu.InitMenu(_ProcessFunc);
}

void menUpdateMenu()
{
	theMenu.UpdateMenu();
}

void menTestMouseMove(CPoint _Cursor)
{
	theMenu.TestMouseMove(_Cursor);
}

void	menTestMouseClick(CPoint _Cursor, UINT	_MouseButton, UINT _ButtonAction)
{
	theMenu.TestMouseClick(_Cursor, _MouseButton, _ButtonAction);
}


void	menCreatePage(DWORD dwID, CGPage	*_NewPage)
{
	theMenu.CreatePage(dwID, _NewPage);
}

void menSetVisiblePage(int PageID)
{
	theMenu.SetCurrentPage(PageID);	
}

void	menReleaseMenu()
{
	theMenu.ReleaseMenu();
}