// GPage.cpp: implementation of the CGPage class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "..\Common\Common.h"
#include "..\Display\Common.h"
#include "..\Input\input1.h"


#include "GPage.h"
#include "GItem.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGPage::CGPage()
{

}

CGPage::~CGPage()
{
	ReleasePage();	
}

void		CGPage::ReleasePage()
{
	CGItem *pItem;
	DXTRACE("Page has %d item(s).", m_Items.GetSize());
	for(int i = 0; i < m_Items.GetSize(); i++) {
		pItem = (CGItem*) m_Items[i];
		DXTRACE("Deleting item. ID: %d\tPointer: 0x%X", pItem->GetID(), int(pItem));		
		SAFE_DELETE(pItem);
	}
	//
	// Remove items from array
	m_Items.RemoveAll();
}

void	CGPage::CreateItem(DWORD dwID, CGItem* _NewItem)
{	
	//
	// Check item pointer
	if(_NewItem == NULL) {
		DXTHROW("Pointer to page is NULL.");
	}
	CGItem* pItem;
	// Check if the item with ID is not already on the page
	for(int i = 0; i < m_Items.GetSize(); i++) {
		pItem = (CGItem*) m_Items[i];
		if(pItem->GetID() == dwID) {
			DXTHROW("Item is already on the page.");	
		}
	}
	// Set some atributes
	_NewItem->SetID(dwID);
	_NewItem->ProcessFunc = ProcessFunc;
	// Add item pointer
	m_Items.Add(_NewItem);
	// Show item pointer
	DXTRACE("Creating item. ID: %d\tPointer: 0x%X", dwID, int(_NewItem));
}

void		CGPage::TestMouseMove(CPoint _Cursor)
{
	CGItem * pItem = NULL;
	CRect rcDestin;
	for(int i = 0; i < m_Items.GetSize(); i++) {
		pItem = (CGItem*) m_Items[i];
		
		pItem->GetDestin(&rcDestin);
		if(pItem && rcDestin.PtInRect(_Cursor)) {

			pItem->ProcessItem(this, 0, IA_MOUSEMOVE);						
		}
		else {
			pItem->ProcessItem(this, 0, IA_NONE);
		}
	}
}

void CGPage::ResetPage() 
{
	CGItem * pItem = NULL;
	for(int i = 0; i < m_Items.GetSize(); i++) {
		pItem = (CGItem*) m_Items[i];
		if(pItem->GetState()->GetID() != BS_DISABLE) {
			pItem->SetState(0);
		}
	}
}

void	CGPage::TestMouseClick(CPoint _Cursor, UINT	_MouseButton, UINT _ButtonAction)
{
	CGItem * pItem = NULL;
	CRect rcDestin;
	for(int i = 0; i < m_Items.GetSize(); i++) {
		pItem = (CGItem*) m_Items[i];
		
		pItem->GetDestin(&rcDestin);
		if(rcDestin.PtInRect(_Cursor)) {

			if(_ButtonAction == BA_UP) {
				pItem->ProcessItem(this, (void*)&_MouseButton, IA_MOUSECLICK_UP);							
			}
			if(_ButtonAction == BA_DOWN) {
				pItem->ProcessItem(this, (void*)&_MouseButton, IA_MOUSECLICK_DOWN);			
			}
		}
		else {
			pItem->ProcessItem(this, (void*)&_MouseButton, IA_NONE);			
		}
	}
}

void		CGPage::UpdatePage()
{
	CGItem *pItem;
	for(int i = 0; i < m_Items.GetSize(); i++) {
		pItem = (CGItem*) m_Items[i];
		pItem->UpdateItem();	
	}
}

