// Sprite.cpp: implementation of the CSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "..\Common\Common.h"
#include "..\Display\Common.h"

#include "Sprite.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSprite::CSprite()
{
	m_pCurState = NULL;
	m_iVelX		= 0;
	m_iVelY		= 0;
	m_ptPos		= CPoint(0, 0);
	m_iCurrentPhase = 0;
	m_iLastUpdate = 0;
	m_rcRealRect = CRect(-1, 0, 0, 0);
}

CSprite::~CSprite()
{	
	m_pCurState = NULL;
}

HRESULT		CSprite::DrawSprite(void)
{
	DWORD dwRet = S_FALSE;
	CRect rcDestin, rcSource;
	//
	// State must be selected
	if(m_pCurState) {		
		//
		// Get rectangles
		GetDestin(&rcDestin);
		GetSource(&rcSource);
		//
		// Draw sprite
		dwRet = disBlt(rcDestin, rcSource, m_pCurState->GetSourceSurface());	
	}
	return dwRet;
}

HRESULT		CSprite::MoveSprite(void)
{
	DWORD dwRet = S_OK;
	//
	// Standard sprite movement
	m_ptPos += CPoint(m_iVelX, m_iVelY);
	//	
	return dwRet;
}

HRESULT		CSprite::ChangeAnimationPhase(void)
{
	DWORD dwRet = S_FALSE;
	//
	// State must be selected
	if(m_pCurState) {
		//
		// Get Current phase that will be changed
		int iPhase  = m_iCurrentPhase;
		//
		// Get time from last update and from windows start
		int newTime = GetTickCount();
		int oldTime = m_iLastUpdate;
		//
		// If is time to update, increment phase
		if((newTime - oldTime) > m_pCurState->GetAnimationSpeed()) {
			//
			// Increment phase
			iPhase++;			
			//
			// If phase is maximum, reset it and do after segment sequention
			if(iPhase >= m_pCurState->GetPhasesCount() - 1) {
						
				iPhase = 0;
				if(m_pCurState->GetDoAfterAnimationEnds() & DAS_POSITION) {					
					m_ptPos = *m_pCurState->GetPosAF();							
				}
				if(m_pCurState->GetDoAfterAnimationEnds() & DAS_STATE) {
					SetState(m_pCurState->GetStateAF());							
				}				
			}
			//
			// Change phase
			m_iCurrentPhase = iPhase;
			//
			// Remember time of this update
			m_iLastUpdate = newTime;
		}		
	}

	return dwRet;
}

void		CSprite::UpdateSprite()
{
	ChangeAnimationPhase();
	MoveSprite();
	DrawSprite();
}

void	CSprite::AddState(CSpriteState * pNewState)
{
	CSpriteState *pState;
	//
	// Check input pointer
	if(!pNewState) {
		DXTRACE("Invalid state poitner has been passed.");
		return;
	}
	// 
	//	Check if the state is not alredy in states array
	for(int i = 0; i < m_arStates.GetSize(); i++) {

		pState = (CSpriteState*) m_arStates[i];

		if(pState->GetID() == pNewState->GetID()) {
			//
			// new state is found in states array
			DXTRACE("State with id: %d is alredy added to this sprite.", pNewState->GetID());
			// Exit
			return;
		}
	}
	//
	// Else add this state
	m_arStates.Add(pNewState);
	//
	// If not is set, set new state as current
	if(!m_pCurState) {
		SetState(pNewState->GetID());		
	}
}

void	CSprite::SetState(DWORD dwState)
{
	CSpriteState *pState;
	// 
	//	Check if the state is in states array
	for(int i = 0; i < m_arStates.GetSize(); i++) {

		pState = (CSpriteState*) m_arStates[i];
		
		if(pState->GetID() == dwState) {
			//
			// State has been found, set it
			m_pCurState = pState;
			return;
		}
	}
	// If state is not valid, set NULL as current state
	m_pCurState = NULL;	
}

BOOL	CSprite::DetectCollision(CRect * pCollisionRect)
{
	CRect FirstRect;
	// Get real rectangle of this sprite
	GetRealRect(&FirstRect);
	// Check intersection both of rectangles
	return FirstRect.IntersectRect(&FirstRect , pCollisionRect);
}

void	CSprite::GetDestin(CRect* rcDestin)
{
	if(m_pCurState || rcDestin) {

		*rcDestin = CRect(	m_ptPos.x, m_ptPos.y, 
							m_ptPos.x + m_pCurState->GetWidth(), 
							m_ptPos.y + m_pCurState->GetHeight()	);
	}
	else {
		*rcDestin = CRect(-1, -1, -1, -1);
	}
}

void	CSprite::GetSource(CRect* rcSource)
{
	if(m_pCurState || rcSource) {

		*rcSource = CRect(	m_iCurrentPhase * m_pCurState->GetWidth(), 
							0, 
							(m_iCurrentPhase + 1) * m_pCurState->GetWidth(), 
							m_pCurState->GetHeight()	);
	}
	else {
		*rcSource = CRect(-1, -1, -1, -1);
	}
}

void	CSprite::SetRealRect(int iLeft, int iTop, int iRight, int iBottom)
{
	m_rcRealRect = CRect(iLeft, iTop, iRight, iBottom);
}

void	CSprite::SetRealRect(CRect * pRealRect)
{
	m_rcRealRect = *pRealRect;
}

void	CSprite::GetRealRect(CRect* pRealRect)
{
	// Check validity of real rect
	if(m_rcRealRect.left == -1 && m_pCurState) {
		//
		// Init standard real rect from size of tile
		m_rcRealRect = CRect(0, 0, m_pCurState->GetWidth(), m_pCurState->GetHeight());
	}
	*pRealRect = m_rcRealRect + m_ptPos;
}
//////////////////////////////////////////////////////////////////////
// CSpriteState Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSpriteState::CSpriteState()
{
	m_dwID = 0;
	m_dwState = 0;
	m_ptPoint = CPoint(0, 0);
	m_dwDoAfterAnimationEnds = 0;
}

CSpriteState::~CSpriteState()
{
	//
	// Release surface
	if(m_dwID != 0) {
		m_surSource.Release();
	}
}

void		CSpriteState::CreateState(DWORD dwID, LPCSTR szSurface, int iAnimationSpeed, int iPhases)
{
	if(m_dwID == 0) {
		// Fill structure
		m_dwID = dwID;
		m_surSource.Create(szSurface);
		m_iAnimationSpeed = iAnimationSpeed;
		m_iPhasesCount = iPhases;
		//
		// Compute dim of one tile
		if(iPhases != 0) {
			m_iWidth = int(m_surSource.Width() / iPhases);
		}
		else {
			m_iWidth = 	m_surSource.Width();
		}
		// Height is same as surface
		m_iHeight = m_surSource.Height();
	}
	else {
		DXTRACE("State is already initialized.");
	}
}

void		CSpriteState::SetDoAfterAnimationEnds(DWORD dwWhatToDo, DWORD dwState, int x, int y)
{
	m_dwDoAfterAnimationEnds = dwWhatToDo;
	m_ptPoint = CPoint(x, y);
	m_dwState = dwState;
}
