// Game.cpp : Defines the entry point for the application.
//

#include	"stdafx.h"
#include	"resource.h"
#include	"..\Common\Common.h"
#include	"..\Display\Common.h"
#include	"..\Input\Input1.h"
#include	"..\Engine\Sprite.h"
#include	"..\Engine\GMenu.h"
//
// Globalni handle na okno
HWND		g_hWnd;
//
// Prototypy globalnich fci
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT CALLBACK ProcessMenu(void *_Page, void *_Item, UINT _Action);

HRESULT WinInit(HINSTANCE hInstance);
void	InitMenu();
void	UpdateFrame();

#define		_S_DATAFILE		_T("Data.dat")
#define		_S_BACKGROUND	_T("\\Graphics\\Backgrounds\\mars800.bmp")

#define		PAGE_MAIN		0
#define		PAGE_EXIT		1
#define		PAGE_OPTIONS	2

#define		BUTTON_STARTGAME	0
#define		BUTTON_OPTIONS		1
#define		BUTTON_CREDITS		2
#define		BUTTON_EXIT			3
#define		LABEL_ROCKET7		4
#define		BUTTON_YES			5
#define		BUTTON_NO			6
#define		LABEL_AREYOUSURE	7
#define		BUTTON_DONE			8
#define		LABEL_CONTROLS		9

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	HRESULT dwRet;
	MSG msg; 

	dwRet = WinInit(hInstance);
	if(dwRet != ERROR_SUCCESS) {
		DXTRACE("Nemohu vytvorit okno aplikace.");
		return dwRet;
	}

	CString csDataPath = setGetDataFilePath(_S_DATAFILE);
	dwRet = stgOpenDataFile3(csDataPath, NULL);
	if(dwRet != ERROR_SUCCESS) {
		DXERR("Cannot open data file due", dwRet);
		return dwRet;
	}

	disInit(g_hWnd, DDFLG_CLIPPER|DDFLG_FULLSCREEN);
	inpCreateDirectInputSystem(hInstance, g_hWnd, disGetResolution());
	disDefineBackground(_S_BACKGROUND, 0);

	InitMenu();
	// Run message loop
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit
                return (int)msg.wParam;
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			UpdateFrame();			
		}
    }	

	return 0;
}

HRESULT WinInit(HINSTANCE hInstance) 
{
    DWORD dwRet;
    WNDCLASSEX wc; 
    //
    // Register the Window Class
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.lpszClassName = "GameWin";
    wc.lpfnWndProc = MainWndProc;
    wc.style = CS_VREDRAW|CS_HREDRAW;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance,MAKEINTRESOURCE(IDI_PROGRAM_ICON));
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wc.lpszMenuName = NULL;    
    wc.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_PROGRAM_ICON));
	wc.cbClsExtra    = 0;
	wc.cbWndExtra	 = 0;
    //
    // Register window class
    if(RegisterClassEx(&wc) == 0) {
        dwRet = GetLastError();
        DXERR("Cannot register the window class due", dwRet); 
        return dwRet; 
    } 
    //
    // Create new main window
    g_hWnd = CreateWindow("GameWin", "Game", 
                            WS_OVERLAPPEDWINDOW, 0, 0, 200, 
                            200, NULL, NULL, hInstance, NULL);
    if(!g_hWnd) {
        dwRet = GetLastError();
        DXERR("Cannot create the window due", dwRet); 
        return dwRet;
    }
    ShowWindow(g_hWnd, SW_SHOW);
    UpdateWindow(g_hWnd);
    dwRet = ERROR_SUCCESS;
    return dwRet;
}

void	InitMenu()
{
	HRESULT dwRet;
	dwRet = menInitMenu(ProcessMenu);
	if(dwRet != ERROR_SUCCESS) {
		DXERR("Cannot init menu due ", dwRet);
		return;
	}
	CGPage	* p_pMain = new CGPage;
	menCreatePage(PAGE_MAIN, p_pMain);

	CGLabel	*l_pLabel1 = new CGLabel;
	l_pLabel1->CreateLabel(IP_CENTER, 130, "\\Graphics\\Menu\\Labels\\rocket7.bmp");
	p_pMain->CreateItem(LABEL_ROCKET7, l_pLabel1);

	CGButton *b_pNewGame = new CGButton;
	b_pNewGame->CreateButton(IP_CENTER, 230, "\\Graphics\\Menu\\Buttons\\startgame_clear.bmp", "\\Graphics\\Menu\\Buttons\\startgame_focus.bmp", "\\Graphics\\Menu\\Buttons\\startgame_press.bmp", "\\Graphics\\Menu\\Buttons\\startgame_dis.bmp");
	p_pMain->CreateItem(BUTTON_STARTGAME, b_pNewGame);
	b_pNewGame->Disable();

	CGButton *b_pOptions = new CGButton;
	b_pOptions->CreateButton(IP_CENTER, 280, "\\Graphics\\Menu\\Buttons\\options_clear.bmp", "\\Graphics\\Menu\\Buttons\\options_focus.bmp","\\Graphics\\Menu\\Buttons\\options_press.bmp");
	p_pMain->CreateItem(BUTTON_OPTIONS, b_pOptions);

	CGButton *b_pCredits = new CGButton;
	b_pCredits->CreateButton(IP_CENTER, 330, "\\Graphics\\Menu\\Buttons\\credits_clear.bmp", "\\Graphics\\Menu\\Buttons\\credits_focus.bmp", "\\Graphics\\Menu\\Buttons\\credits_press.bmp", "\\Graphics\\Menu\\Buttons\\credits_dis.bmp");
	p_pMain->CreateItem(BUTTON_CREDITS, b_pCredits);
	b_pCredits->Disable();

	CGButton *b_pExit = new CGButton;
	b_pExit->CreateButton(IP_CENTER, 380, "\\Graphics\\Menu\\Buttons\\exitgame_clear.bmp", "\\Graphics\\Menu\\Buttons\\exitgame_focus.bmp", "\\Graphics\\Menu\\Buttons\\exitgame_press.bmp");
	p_pMain->CreateItem(BUTTON_EXIT, b_pExit);


	CGPage	* p_pExit = new CGPage;
	menCreatePage(PAGE_EXIT, p_pExit);

	CGLabel	*l_pLabel2 = new CGLabel;
	l_pLabel2->CreateLabel(IP_CENTER, 130, "\\Graphics\\Menu\\Labels\\areyousure1.bmp");
	p_pExit->CreateItem(LABEL_AREYOUSURE, l_pLabel2);

	CGButton *b_pYes = new CGButton;
	b_pYes->CreateButton(IP_CENTER, 230, "\\Graphics\\Menu\\Buttons\\yes_clear.bmp", "\\Graphics\\Menu\\Buttons\\yes_focus.bmp", "\\Graphics\\Menu\\Buttons\\yes_press.bmp");
	p_pExit->CreateItem(BUTTON_YES, b_pYes);

	CGButton *b_pNo = new CGButton;
	b_pNo->CreateButton(IP_CENTER, 280, "\\Graphics\\Menu\\Buttons\\no_clear.bmp", "\\Graphics\\Menu\\Buttons\\no_focus.bmp", "\\Graphics\\Menu\\Buttons\\no_press.bmp");
	p_pExit->CreateItem(BUTTON_NO, b_pNo);

	
	CGPage	* p_pOptions = new CGPage;
	menCreatePage(PAGE_OPTIONS, p_pOptions);

	CGButton *b_pDone = new CGButton;
	b_pDone->CreateButton(IP_CENTER, 450, "\\Graphics\\Menu\\Buttons\\done_clear.bmp", "\\Graphics\\Menu\\Buttons\\done_focus.bmp", "\\Graphics\\Menu\\Buttons\\done_press.bmp");
	p_pOptions->CreateItem(BUTTON_DONE, b_pDone);

	CGLabel	*l_pLabel3 = new CGLabel;
	l_pLabel3->CreateLabel(IP_CENTER, 80, "\\Graphics\\Menu\\Labels\\controls.bmp");
	p_pOptions->CreateItem(LABEL_CONTROLS, l_pLabel3);


}

LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch(msg) {
	case WM_SETCURSOR:
		SetCursor(NULL);		
		return 0L;
	case WM_SYSCOMMAND:
        // Prevent moving/sizing and power loss in fullscreen mode
        switch( wParam )
        {
            case SC_MOVE:
            case SC_SIZE:
            case SC_MAXIMIZE:
            case SC_KEYMENU:
            case SC_MONITORPOWER:
                return 1;                
        }
		break;
	case WM_DESTROY:
        // Cleanup and close the app     
		menReleaseMenu();
        PostQuitMessage( 0 );
        return 0L;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

void	UpdateFrame()
{
	disUpdateBackground();

	inpProcessInput();

	// Pri stisknuti klavesy Esc ukoncime aplikaci
	if(inpIsKeyDown(DIK_ESCAPE, FALSE)) {
		PostMessage(g_hWnd, WM_DESTROY, 0, 0);
	}
	menTestMouseMove(inpGetCursor());
	if(inpIsLButtonDown()) {
		menTestMouseClick(inpGetCursor(), LEFT_MOUSE_BUTTON, BA_DOWN);
	} 
	if(inpIsLButtonUp()) {	
		menTestMouseClick(inpGetCursor(), LEFT_MOUSE_BUTTON, BA_UP);
	}	

	menUpdateMenu();
	inpUpdateCursor();
	

	disPresent();

	
}

HRESULT CALLBACK ProcessMenu(void *_Page, void *_Item, UINT _Action)
{
	CGPage*   Page	  = (CGPage*)  _Page;
	CGItem*   Item    = (CGItem*)  _Item;

	if(!Page) {
		return ERROR_SUCCESS;
	}

	ASSERT(Page);
	ASSERT(Item);
	
	if(Item->GetState()->GetID() != BS_DISABLE) {
		switch(Page->GetID()) {
		//////////////prvni stranka MAIN
		case PAGE_MAIN:
			switch(Item->GetID()) {
			///////////tlacitko 1 START GAME
			case BUTTON_STARTGAME:				
				//none handling
				break;
			///////////tlacitko 2 // OPTIONS
			case BUTTON_OPTIONS:
				if(_Action == IA_MOUSECLICK_UP) {
					menSetVisiblePage(PAGE_OPTIONS);					
				}
				break;			
			///////////tlacitko 4
			case BUTTON_EXIT: // EXIT GAME
				if(_Action == IA_MOUSECLICK_UP) {	
					menSetVisiblePage(PAGE_EXIT);										
				}			
				break;
			}
			break;
		///////////druha stranka EXIT
		case PAGE_EXIT:
			switch(Item->GetID()) {
			//////prvni tlacitko YES
			case BUTTON_YES:
				if(_Action == IA_MOUSECLICK_UP) {
					PostMessage(g_hWnd, WM_CLOSE, 0, 0);
				}
				break;
			//////druhy tlacitko NO
			case BUTTON_NO:
				if(_Action == IA_MOUSECLICK_UP) {
					menSetVisiblePage(PAGE_MAIN);										
				}
				break;
			}
			break;		
		case PAGE_OPTIONS:
			switch(Item->GetID()) {
			case BUTTON_DONE:
				if(_Action == IA_MOUSECLICK_UP) {
					menSetVisiblePage(PAGE_MAIN);
				}
				break;
			}
			break;	
		}
	}
	return 0;
}