// Cursor.cpp: implementation of the CCursor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Common\Common.h"
#include "..\Display\Common.h"

#include "Cursor.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCursor::CCursor()
{
	m_bInit = FALSE;
	//
	// Indicates static cursor
	m_nPhases = 0;
}

CCursor::~CCursor()
{
	m_bInit = FALSE;
}

/*
 *	First version of creation function
 *	Create only static cursor from bitmap
 */
HRESULT	CCursor::Create(CString csIntPath, int cx, int cy)
{
	DWORD dwRet = 1;
	if(!m_bInit) {
		
		m_iWidth = cx;
		m_iHeight = cy;
		m_csIntPath = csIntPath;

		dwRet = m_surCursor.Create(csIntPath, DDFLG_COLORKEY);
		if(dwRet != ERROR_SUCCESS) {
			DXERR("Cannot create surface for cursor due", dwRet);
			return dwRet;
		}
		m_bInit = true;
		dwRet = ERROR_SUCCESS;
	}
	return dwRet;
}

/*
 *  Second version of creation function
 *  Create animate cursor
 */
HRESULT	CCursor::Create(CString csIntPath, int cx, int cy, int iSpeed)
{
	DWORD dwRet = 1;
	if(!m_bInit) {
		//
		// Call standard creation
		dwRet = Create(csIntPath, cx, cy);
		if(dwRet == ERROR_SUCCESS) {
			m_iPhase = 0;			
			m_iSpeed  = iSpeed;
			//
			// Compute number of phases
			m_nPhases = (int)m_surCursor.Width() / cx;
			// 
			// Check validity of phases
			if((m_nPhases * cx) > (int)m_surCursor.Width()) m_nPhases--;
			//
			// Ok
			m_bInit = true;
		}		
	}
	return dwRet;
}

/*
 *		Draw cursor
 */
HRESULT		CCursor::Update(CPoint ptCursor)
{
	CRect rcDest, rcSrc;
	DWORD dwRet = 1, newTime;
	if(m_bInit) {		
		//
		// For static cursors
		if(m_nPhases == 0) {
			//
			// One static place
			rcSrc.top = 0;
			rcSrc.left = 0;
			rcSrc.right = m_iWidth;
			rcSrc.bottom = m_iHeight;
		}
		//
		// Compute move source rectangle
		else {
			//
			// Compute phase, increment phase if is time
			newTime = GetTickCount();
			//
			// Get time from last update
			if((newTime - m_dwOldTime) > (DWORD)m_iSpeed) {
				//
				// If it right time, increment phase
				m_iPhase++;
				//
				// Check to overflow phases
				if(m_iPhase == m_nPhases) {
					m_iPhase = 0;
				}
				//
				// Save time of last update
				m_dwOldTime = newTime;
			}
			//
			// Compute rectangle from current phase
			//
			// Static vertical postion
			rcSrc.top = 0;
			rcSrc.bottom = m_iHeight;
			// Variable horizontal pos.
			rcSrc.left = m_iPhase * m_iWidth;
			rcSrc.right = rcSrc.left + m_iWidth;			
		}		
		//
		// Destionation rectangle is same for both cursors
		rcDest.left = ptCursor.x;
		rcDest.top = ptCursor.y;
		rcDest.bottom = rcDest.top + m_iHeight;
		rcDest.right =  rcDest.left + m_iWidth;
		//
		// Blit cursor to back surface		
		dwRet = disBlt(rcDest, rcSrc, &m_surCursor);
	}
	return dwRet;
}
