// Input1.cpp: implementation of the CInput class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#define   INPUT_API __declspec(dllexport)

#include "..\Common\Common.h"
#include "..\Display\Common.h"

#include "Input1.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// Globalni objekt DI
CInput	g_theInput;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInput::CInput() 
{
	m_bInit = FALSE;
	m_bShowCursor = TRUE;

	m_lpDI = NULL;
	m_lpDIDKeyboard = NULL;
	m_lpDIDMouse = NULL;
}

CInput::~CInput()
{
	Clean();
}

void CInput::Clean()
{
	if(m_bInit) {

		if(m_lpDIDKeyboard) 
			m_lpDIDKeyboard->Release();

		if(m_lpDIDMouse) 
			m_lpDIDMouse->Release();

		if(m_lpDI) 
			m_lpDI->Release();

		m_bInit = FALSE;
	}
}

HRESULT CInput::CreateDirectInputSystem(HINSTANCE hInst, HWND hWnd, CSize csResolution)
{
	DXTRACE("Initializing direct input...");
	HRESULT dwRet = 1;
	//
	// You cannot call this function twice
	if(!m_bInit) {
		//
		// Creation of direct input object
		dwRet = DirectInput8Create(hInst, DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID**)&m_lpDI, NULL);		
		if(dwRet != DI_OK) {
			DXERR("Cannot create direct input object due", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Create keyboard object
		dwRet = m_lpDI->CreateDevice(GUID_SysKeyboard, &m_lpDIDKeyboard, NULL);		
		if(dwRet != DI_OK) {
			DXERR("Cannot create keyboard object due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Create mouse object
		dwRet = m_lpDI->CreateDevice(GUID_SysMouse, &m_lpDIDMouse, NULL);		
		if(dwRet != DI_OK) {
			DXERR("Cannot create mouse object due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Set data format for both devices
		dwRet = m_lpDIDMouse->SetDataFormat(&c_dfDIMouse);		
		if(dwRet != DI_OK) {
			DXERR("Cannot set data format for mouse due ", dwRet);
			Clean();
			return dwRet;
		}
		dwRet = m_lpDIDKeyboard->SetDataFormat(&c_dfDIKeyboard);		
		if(dwRet != DI_OK) {
			DXERR("Cannot set data format for keyboard due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Set cooperative level for both devices
		dwRet = m_lpDIDKeyboard->SetCooperativeLevel(hWnd, DISCL_FOREGROUND|DISCL_NONEXCLUSIVE);		
		if(dwRet != DI_OK) {
			DXERR("Cannot set cooperative level for keyboard due ", dwRet);
			Clean();
			return dwRet;
		}	
		dwRet = m_lpDIDMouse->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE|DISCL_FOREGROUND);		
		if(dwRet != DI_OK) {
			DXERR("Cannot set cooperative level for mouse due ", dwRet);
			Clean();
			return dwRet;
		}
		//	
		// Acquire mouse and keyboard
		dwRet = m_lpDIDKeyboard->Acquire();		
		if(dwRet != DI_OK) {
			DXERR("Cannot acquire keyboard due ", dwRet);
			Clean();
			return dwRet;
		}
		
		dwRet = m_lpDIDMouse->Acquire();		
		if(dwRet != DI_OK) {
			DXERR("Cannot acquire mouse due ", dwRet);
			Clean();
			return dwRet;
		}		
		//
		m_csResolution = csResolution;
		//
		// Create new cursor
		dwRet = m_theCursor.Create(_S_CURSOR, 32, 32, 50);
		if(dwRet != DI_OK) {
			DXERR("Cannot create cursor due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Pocatecni vystredeni kurzoru
		m_ptCursor = CPoint(m_csResolution.cx/2, m_csResolution.cy/2);

		m_bInit = TRUE;
	}
	return dwRet;
}

HRESULT	CInput::ProcessInput()
{
	HRESULT dwRet = 1;
	if(m_bInit) {
		//
		// Get state of keyboard and save it to internal structure (array)
		m_lpDIDKeyboard->GetDeviceState(sizeof(m_arKeyboard), &m_arKeyboard);
		//
		// Update cursor if is showed
		if(m_bShowCursor) {
			//
			// Get relative position of cursor.
			// Save state of mouse.
			m_lpDIDMouse->GetDeviceState(sizeof(m_MouseState) , &m_MouseState);			
			//
			// Add or substract position of cursor
			MoveCursor(m_MouseState.lX, m_MouseState.lY);
			//
			// Bounds cursor on the screen
			if(m_ptCursor.x < 0) m_ptCursor.x = 0;
			if(m_ptCursor.x > m_csResolution.cx) m_ptCursor.x = m_csResolution.cx;
			if(m_ptCursor.y < 0) m_ptCursor.y = 0;
			if(m_ptCursor.y > m_csResolution.cy) m_ptCursor.y = m_csResolution.cy;		
		}		
	}
	return dwRet;
}

HRESULT CInput::UpdateCursor()
{
	HRESULT dwRet = 1;
	if(m_bInit && m_bShowCursor) {

		m_theCursor.Update(m_ptCursor);	
	}
	return dwRet;
}

BOOL	CInput::IsRButtonDown(BOOL _Handling)
{
	if(m_bInit) {
		//
		// Checking left mouse button
		if(m_MouseState.rgbButtons[MOUSEBUTTON_RIGHT]) {
			//
			// If handling is not set, set it
			if(_Handling) {
				//
				// User press mouse button...handling activated
				if(!m_IsHandled[MOUSEBUTTON_RIGHT]) {	
					m_IsHandled[MOUSEBUTTON_RIGHT] = TRUE;					
					return TRUE;
				}
				else {
					//
					// Return false because handling is already set.
					return FALSE;				
				}
			}
			else {
				//
				// User does not want hadling, simply return TRUE
				return TRUE;					
			}
			
		}
		else {
			//
			// Reset handling because mouse button is up
			m_IsHandled[MOUSEBUTTON_RIGHT] = FALSE;
			//			
			// Button is up so return FALSE
			return FALSE;	
		}
	}
	//
	// DirectInput is not initialized, return FALSE
	return FALSE;	
}
BOOL	CInput::IsLButtonDown(BOOL _Handling)
{
	if(m_bInit) {
		//
		// Checking left mouse button
		if(m_MouseState.rgbButtons[MOUSEBUTTON_LEFT]) {
			//
			// If handling is not set, set it
			if(_Handling) {
				//
				// User press mouse button...handling activated
				if(!m_IsHandled[MOUSEBUTTON_LEFT]) {	
					m_IsHandled[MOUSEBUTTON_LEFT] = TRUE;
					m_bButtonDown = TRUE;
					return TRUE;
				}
				else {
					//
					// Return false because handling is already set.
					return FALSE;				
				}
			}
			else {
				//
				// User does not want hadling, simply return TRUE
				m_bButtonDown = TRUE;
				return TRUE;					
			}
			
		}
		else {
			//
			// Reset handling because mouse button is up
			m_IsHandled[MOUSEBUTTON_LEFT] = FALSE;			
			//			
			// Button is up so return FALSE
			return FALSE;	
		}
	}
	//
	// DirectInput is not initialized, return FALSE
	return FALSE;
}

BOOL	CInput::IsRButtonUp()
{
	if(m_bInit) {
		if(m_bButtonDown && !m_MouseState.rgbButtons[MOUSEBUTTON_RIGHT]) {
			m_bButtonDown = FALSE;
			return TRUE;
		}
		
		//return (m_MouseState.rgbButtons[MOUSEBUTTON_RIGHT]) ? FALSE : TRUE;
	}
	
	return FALSE;
}

BOOL	CInput::IsLButtonUp()
{
	if(m_bInit) {
		if(m_bButtonDown && !m_MouseState.rgbButtons[MOUSEBUTTON_LEFT]) {
			m_bButtonDown = FALSE;
			return TRUE;
		}
		//return (m_MouseState.rgbButtons[MOUSEBUTTON_LEFT]) ? FALSE : TRUE;
	}
	
	return FALSE;	
}

BOOL	CInput::IsKeyDown(int Key, BOOL bHandle)
{
	if(m_bInit) {
		//
		// Check validity of input param
		if(Key >= 256) {		
			return FALSE;
		}
		//
		// Checking specified key
		if(KEYDOWN(m_arKeyboard, Key)) {
			if(bHandle) {
				//
				// User press key button...handling activated
				if(!m_IsHandled[Key]) {
					m_IsHandled[Key] = TRUE;
					return TRUE;
				}
				else {
					//
					// Return false because handling is already set.
					return FALSE;				
				}
			}
			//
			// User does not want handling, simply return TRUE
			else {
				return TRUE;
			}			
		}
		//
		// Key is up, reset handling, return FALSE
		else {
			m_IsHandled[Key] = FALSE;
			return FALSE;
			
		}
	}
	//
	// DirectInput is not initialized, return FALSE
	return FALSE;	
}


HRESULT CInput::RestoreDevices()
{
    DWORD dwRet = 1; // INPUT IS NOT INIT
    if(m_bInit) {
        dwRet = m_lpDIDKeyboard->Acquire();
        if(dwRet != DI_OK) {
            DXERR("Cannot reacquire the keyboard due ", dwRet);
            return dwRet;
        }
        dwRet = m_lpDIDMouse->Acquire();
        if(dwRet != DI_OK) {
            DXERR("Cannot reacquire the mouse due ", dwRet);
            return dwRet;
        }
    }
    return dwRet;
}



//
// Exportovane funkce
HRESULT	inpCreateDirectInputSystem(HINSTANCE hInst, HWND hWnd, CSize csResolution)
{
	return g_theInput.CreateDirectInputSystem(hInst, hWnd, csResolution);
}

HRESULT	inpProcessInput()
{
	return g_theInput.ProcessInput();
}

HRESULT	inpUpdateCursor()
{
	return g_theInput.UpdateCursor();
}

void	inpShowCursor(BOOL bShow)
{
	g_theInput.ShowCursor(bShow);
}

BOOL	inpIsRButtonDown(BOOL _Handling)
{
	return g_theInput.IsRButtonDown(_Handling);
}

BOOL	inpIsLButtonDown(BOOL _Handling)
{
	return g_theInput.IsLButtonDown(_Handling);
}

BOOL	inpIsRButtonUp()
{
	return g_theInput.IsRButtonUp();
}

BOOL	inpIsLButtonUp()
{
	return g_theInput.IsLButtonUp();
}

BOOL	inpIsKeyDown(int Key, BOOL bHandle)
{
	return g_theInput.IsKeyDown(Key, bHandle);
}

HRESULT inpRestoreDevices()
{
	return g_theInput.RestoreDevices();
}

CPoint	inpGetCursor()
{
	return g_theInput.GetCursor();
}
